@@ -215,6 +215,7 @@ class LutronXmlDbParser(object):
     self._lutron = lutron
     self._xml_db_str = xml_db_str
     self.areas = []
+    self._occupancy_groups = {}
     self.project_name = None
 
   def parse(self):
@@ -237,6 +238,19 @@ class LutronXmlDbParser(object):
     # identifiers that won't change over time.
     self._lutron.set_guid(root.find('GUID').text)
 
+    # Parse Occupancy Groups
+    # OccupancyGroups are referenced by entities in the rest of the XML.  The
+    # current structure of the code expects to go from areas -> devices ->
+    # other assets and attributes.  Here we index the groups to be bound to
+    # Areas later.
+    groups = root.find('OccupancyGroups')
+    for group_xml in groups.getiterator('OccupancyGroup'):
+      group = self._parse_occupancy_group(group_xml)
+      if group.group_number:
+        self._occupancy_groups[group.group_number] = group
+      else:
+        _LOGGER.warning("Occupancy Group has no number.  XML: %s", group_xml)
+
     # First area is useless, it's the top-level project area that defines the
     # "house". It contains the real nested Areas tree, which is the one we want.
     top_area = root.find('Areas').find('Area')
@@ -250,10 +264,15 @@ class LutronXmlDbParser(object):
   def _parse_area(self, area_xml):
     """Parses an Area tag, which is effectively a room, depending on how the
     Lutron controller programming was done."""
+    occupancy_group_id = area_xml.get('OccupancyGroupAssignedToID')
+    occupancy_group = self._occupancy_groups.get(occupancy_group_id)
+    area_name = area_xml.get('Name')
+    if not occupancy_group:
+      _LOGGER.warning("Occupancy Group not found for Area: %s; ID: %s", area_name, occupancy_group_id)
     area = Area(self._lutron,
-                name=area_xml.get('Name'),
+                name=area_name,
                 integration_id=int(area_xml.get('IntegrationID')),
-                occupancy_group_id=area_xml.get('OccupancyGroupAssignedToID'))
+                occupancy_group=occupancy_group)
     for output_xml in area_xml.find('Outputs'):
       output = self._parse_output(output_xml)
       area.add_output(output)
@@ -289,13 +308,17 @@ class LutronXmlDbParser(object):
   def _parse_output(self, output_xml):
     """Parses an output, which is generally a switch controlling a set of
     lights/outlets, etc."""
-    output = Output(self._lutron,
-                    name=output_xml.get('Name'),
-                    watts=int(output_xml.get('Wattage')),
-                    output_type=output_xml.get('OutputType'),
-                    integration_id=int(output_xml.get('IntegrationID')),
-                    uuid=output_xml.get('UUID'))
-    return output
+    output_type = output_xml.get('OutputType')
+    kwargs = {
+      'name': output_xml.get('Name'),
+      'watts': int(output_xml.get('Wattage')),
+      'output_type': output_type,
+      'integration_id': int(output_xml.get('IntegrationID')),
+      'uuid': output_xml.get('UUID')
+    }
+    if output_type == 'SYSTEM_SHADE':
+      return Shade(self._lutron, **kwargs)
+    return Output(self._lutron, **kwargs)
 
   def _parse_keypad(self, keypad_xml, device_group):
     """Parses a keypad device (the Visor receiver is technically a keypad too)."""
@@ -366,6 +389,15 @@ class LutronXmlDbParser(object):
                         integration_id=int(sensor_xml.get('IntegrationID')),
                         uuid=sensor_xml.get('UUID'))
 
+  def _parse_occupancy_group(self, group_xml):
+    """Parses an Occupancy Group object.
+
+    These are defined outside of the areas in the XML.  Areas refer to these
+    objects by ID.
+    """
+    return OccupancyGroup(self._lutron,
+                          group_number=group_xml.get('OccupancyGroupNumber'),
+                          uuid=group_xml.get('UUID'))
 
 class Lutron(object):
   """Main Lutron Controller class.
@@ -405,6 +437,10 @@ class Lutron(object):
   def guid(self):
     return self._guid
 
+  @property
+  def name(self):
+    return self._name
+
   def subscribe(self, obj, handler):
     """Subscribes to status updates of the requested object.
 
@@ -716,6 +752,28 @@ class Output(LutronEntity):
     return self.type != 'NON_DIM' and not self.type.startswith('CCO_')
 
 
+class Shade(Output):
+  """This is the output entity for shades in Lutron universe."""
+  _ACTION_RAISE = 2
+  _ACTION_LOWER = 3
+  _ACTION_STOP = 4
+
+  def start_raise(self):
+    """Starts raising the shade."""
+    self._lutron.send(Lutron.OP_EXECUTE, Output._CMD_TYPE, self._integration_id,
+        Output._ACTION_RAISE)
+
+  def start_lower(self):
+    """Starts lowering the shade."""
+    self._lutron.send(Lutron.OP_EXECUTE, Output._CMD_TYPE, self._integration_id,
+        Output._ACTION_LOWER)
+
+  def stop(self):
+    """Starts raising the shade."""
+    self._lutron.send(Lutron.OP_EXECUTE, Output._CMD_TYPE, self._integration_id,
+        Output._ACTION_STOP)
+
+
 class KeypadComponent(LutronEntity):
   """Base class for a keypad component such as a button, or an LED."""
 
@@ -1104,19 +1162,29 @@ class OccupancyGroup(LutronEntity):
     """
     OCCUPANCY = 1
 
-  def __init__(self, lutron, area, uuid):
-    super(OccupancyGroup, self).__init__(lutron, 'Occ {}'.format(area.name), uuid)
+  def __init__(self, lutron, group_number, uuid):
+    super(OccupancyGroup, self).__init__(lutron, None, uuid)
+    self._area = None
+    self._group_number = group_number
+    self._integration_id = None
+    self._state = None
+    self._query_waiters = _RequestHelper()
+
+  def _bind_area(self, area):
     self._area = area
     self._integration_id = area.id
-    self._state = None
     self._lutron.register_id(OccupancyGroup._CMD_TYPE, self)
-    self._query_waiters = _RequestHelper()
 
   @property
   def id(self):
     """The integration id"""
     return self._integration_id
 
+  @property
+  def group_number(self):
+    """The OccupancyGroupNumber"""
+    return self._group_number
+
   @property
   def name(self):
     """Return the name of this OccupancyGroup, which is 'Occ' plus the name of the area."""
@@ -1164,15 +1232,16 @@ class OccupancyGroup(LutronEntity):
 
 class Area(object):
   """An area (i.e. a room) that contains devices/outputs/etc."""
-  def __init__(self, lutron, name, integration_id, occupancy_group_id):
+  def __init__(self, lutron, name, integration_id, occupancy_group):
     self._lutron = lutron
     self._name = name
     self._integration_id = integration_id
-    self._occupancy_group_id = occupancy_group_id
-    self._occupancy_group = None
+    self._occupancy_group = occupancy_group
     self._outputs = []
     self._keypads = []
     self._sensors = []
+    if occupancy_group:
+      occupancy_group._bind_area(self)
 
   def add_output(self, output):
     """Adds an output object that's part of this area, only used during
@@ -1188,9 +1257,6 @@ class Area(object):
     """Adds a motion sensor object that's part of this area, only used during
     initial parsing."""
     self._sensors.append(sensor)
-    if not self._occupancy_group:
-      # TODO: add the uuid for the occupancy group
-      self._occupancy_group = OccupancyGroup(self._lutron, self, None)
 
   @property
   def name(self):
